/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 */

package org.objectweb.dream.dreamannotation.generator.template.lifecycle;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.dream.control.lifecycle.BeforeFirstStartLifeCycle;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;

import spoon.reflect.code.CtExpression;
import spoon.reflect.code.CtInvocation;
import spoon.reflect.code.CtVariableAccess;
import spoon.reflect.declaration.CtField;
import spoon.reflect.declaration.CtMethod;
import spoon.reflect.reference.CtExecutableReference;
import spoon.reflect.reference.CtFieldReference;
import spoon.template.Local;
import spoon.template.Parameter;
import spoon.template.Template;

/**
 * A template code to manage the LifeCycle annotation. This template introdudes
 * the implementation of the LifeCycle controller and then call the annotated
 * method in the start or stop method (depending on the parameter of the
 * annotation).
 * 
 * @author Nicolas Pessemier <Nicolas.Pessemier@lifl.fr>
 */
public class DreamLifeCycleTemplateFirstStart implements LifeCycleController,
		Template, BeforeFirstStartLifeCycle {

	@SuppressWarnings("unchecked")
	@Local
	public DreamLifeCycleTemplateFirstStart(CtMethod<?> methodBeforeFirstStart,
			CtField<?> weaveableC, Class<?> processedClass) {
		// method before first start
		_methodCallBeforeFirstStart_ = methodBeforeFirstStart.getFactory()
				.Core().createInvocation();
		// _methodCallBeforeFirstStart_.setTarget(methodBeforeFirstStart.getFactory()
		// .Code().createThisAccess(
		// methodBeforeFirstStart.getFactory().Type().createReference(
		// methodBeforeFirstStart.getParent(CtClass.class)
		// .getActualClass())));

		// argument managmenent
		// creation of the field reference
		// to force this.weaveable and not SuperClass.this.weavable
		CtFieldReference<?> cfr = weaveableC.getFactory().Field()
				.createReference(weaveableC);
		cfr.setDeclaringType((methodBeforeFirstStart.getFactory().Type()
				.createReference(processedClass)));
		CtVariableAccess<?> cva = weaveableC.getFactory().Code()
				.createVariableAccess(cfr, false);
		List<CtExpression<?>> cte = new ArrayList<CtExpression<?>>();
		cte.add((CtExpression<?>) cva);
		_methodCallBeforeFirstStart_.setArguments(cte);

		_methodCallBeforeFirstStart_
				.setExecutable((CtExecutableReference) methodBeforeFirstStart
						.getReference());

		this.__weaveableC_ = weaveableC.getSimpleName();
	}

	@Parameter
	CtInvocation<?> _methodCallBeforeFirstStart_;

	@Local
	// A Local variable which is the reference to the singeton field
	Component _weaveableC_;

	@Parameter("_weaveableC_")
	// The name of the singleton field
	String __weaveableC_;

	/**
	 * This attribute give the state of Dream Component. <br>
	 * the fcSstate value is {@link LifeCycleController#STARTED}
	 * {@link LifeCycleController#STOPPED}
	 */
	private String _fcState = LifeCycleController.STOPPED;

	/**
	 * True if it is the first start of the component
	 */
	private boolean _firstStart = true;

	/**
	 * (non-Javadoc) Method automatically generated with Spoon
	 * 
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return _fcState;
	}

	/**
	 * (non-Javadoc) Method automatically generated with Spoon
	 * 
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {
		try {
			_fcState = LifeCycleController.STARTED;
			if (_firstStart) {
				_beforeFirstStart(_weaveableC_);
				_firstStart = false;
			}
		} catch (Exception e) {
			throw new IllegalLifeCycleException(e.getMessage());
		}
	}

	/**
	 * (non-Javadoc) Method automatically generated with Spoon
	 * 
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {
		_fcState = LifeCycleController.STOPPED;
	}

	/**
	 * This method is called the first time the component is started.
	 * 
	 * @param componentItf
	 *            the {@link Component }interface of this component.
	 */
	public void _beforeFirstStart(Component componentItf)
			throws IllegalLifeCycleException {
		try {
			_methodCallBeforeFirstStart_.S();
		} catch (Exception e) {
			throw new IllegalLifeCycleException(e.getMessage());
		}
	}
}
