/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 */
package org.objectweb.dream.dreamannotation.generator.template.monolog;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.dream.InitializationException;
import org.objectweb.dream.control.logger.Loggable;
import org.objectweb.dream.control.logger.LoggerControllerRegister;
import org.objectweb.dream.dreamannotation.DreamMonolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import spoon.reflect.declaration.CtField;
import spoon.template.Local;
import spoon.template.Parameter;
import spoon.template.Template;

/**
 * A template to manage the reference to the "dream" logger. It also implements the Loggable interace and the _initLogger() methode
 * 
 * @author Pierre Garcia <Pierre.Garcia@inrialpes.fr>
 * 
 */
public class DreamMonologTemplate implements Template, Loggable {

	@Local
	public DreamMonologTemplate(CtField<?> monologField, CtField<?> serviceField, DreamMonolog logAnnotation) {
		this.__monologField_ = monologField.getSimpleName();
    this.__serviceField_ = serviceField.getSimpleName();
	}

	@Local
	Logger _monologField_;

	// the simple name of the field
	@Parameter("_monologField_")
	String __monologField_;
  
  @Local
  Component _serviceField_;

  // the simple name of the service field
  @Parameter("_serviceField_")
  String __serviceField_;

  /**
   * @see Loggable#setLogger(String, Logger)
   */
  public void setLogger(String name, Logger logger)
  {
    if (name.equals("impl"))
    {
      this._monologField_ = logger;
    }
  }
  
  /**
   * Initializes the logger of this component. This method is called during initialization of
   * the component controllers. <br>
   *
   * @throws InitializationException if an error occurs.
   * 
   */
  protected void _initLogger() throws InitializationException
  {
    try
    {
      LoggerControllerRegister lcr = (LoggerControllerRegister) _serviceField_
          .getFcInterface("/logger-controller-register");
      lcr.register("impl", this);
      _monologField_.log(BasicLevel.DEBUG, "Component initialized.");
    }
    catch (NoSuchInterfaceException e)
    {
      // ignore
    }
  }
}
