#!/bin/sh

RepScript=`dirname $0`
RepCourant=`pwd`

usage()
{
	echo "The shutdown.sh script is used to shutdown Petals."
    echo "usage : shutdown.sh [-h|-y]"
    echo "     -h : prints the help message"
    echo "     -y : no confirmation is asked"
    exit 0;
}

# arguments analysis
Confirmed=false
while getopts :hv:yv options
do
        case ${options} in
                h)
                        usage
                        ;;
                y)
                        Confirmed=true
                        ;;
        esac
done


echo "Setting Petals environment"
######################################
#   ENV VARIABLES
######################################


if [ -z "$JAVA_HOME" ]
then
        echo "The JAVA_HOME variable is not set. Please initialise it in your environment."
        exit 1
fi

if [ -z $PETALS_HOME ]
then
	PETALS_HOME=$RepCourant/$RepScript/../
fi

if [ ! -f $PETALS_HOME/conf/server.properties ]
then
	echo "The PETALS_HOME environment variable cannot be set correctly. Please set it in your environment with the correct value."
	exit 2
fi




PATH=$JAVA_HOME/bin/:$PATH

export PATH

#####################################

cd $PETALS_HOME/bin


if [ "$Confirmed" = "false" ]
then
	echo "Are you sure to remove this PEtALS instance from your PEtALS network (Y/n) ?"
	echo "    - all services assembly of this instance will be uninstalled,"
	echo "    - all components of this instance will be uninstalled,"
	echo "    - this PEtALS instance will be unregistered."
	read ANSWER
	if [ "$ANSWER" = "Y" ]
	then
		Confirmed=true
	fi
fi

if [ "$Confirmed" = "true" ]
then
	java -Xmx1024m -jar server.jar shutdown
else
	echo "Shutdown canceled."
fi

exit 0
